@@ -129,8 +129,6 @@ static gboolean g_buffered_input_stream_truncate            (GSeekable       *se
 							     GCancellable    *cancellable,
 							     GError         **error);
 
-static void     g_buffered_input_stream_finalize            (GObject         *object);
-
 static void compact_buffer (GBufferedInputStream *stream);
 
 G_DEFINE_TYPE_WITH_CODE (GBufferedInputStream,
@@ -641,7 +639,7 @@ compact_buffer (GBufferedInputStream *stream)
 
   current_size = priv->end - priv->pos;
 
-  g_memmove (priv->buffer, priv->buffer + priv->pos, current_size);
+  memmove (priv->buffer, priv->buffer + priv->pos, current_size);
 
   priv->pos = 0;
   priv->end = current_size;
