@@ -23,5 +23,5 @@
 
 DATADIR = "${prefix}/share"
 DATADIR = DATADIR.replace(
-    "${prefix}", "/home/mclasen/gnome")
-VERSION = "2.39.2"
+    "${prefix}", "/usr/local")
+VERSION = "2.39.3"
