@@ -19,6 +19,8 @@
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
 
+#include "config.h"
+
 #include "gdbusmenumodel.h"
 
 #include "gmenumodel.h"
@@ -29,6 +31,7 @@
  * SECTION:gdbusmenumodel
  * @title: GDBusMenuModel
  * @short_description: A D-Bus GMenuModel implementation
+ * @include: gio/gio.h
  * @see_also: <link linkend="gio-GMenuModel-exporter">GMenuModel Exporter</link>
  *
  * #GDBusMenuModel is an implementation of #GMenuModel that can be used
@@ -381,6 +384,7 @@ g_dbus_menu_group_unref (GDBusMenuGroup *group)
 
       g_hash_table_remove (group->path->groups, GINT_TO_POINTER (group->id));
       g_hash_table_unref (group->proxies);
+      g_hash_table_unref (group->menus);
 
       g_dbus_menu_path_unref (group->path);
 
