@@ -25,12 +25,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef _WIN32
-#include <io.h>
-#endif
 
 #include "giotypes.h"
 #include "gioerror.h"
@@ -51,6 +45,13 @@
 #include "ginputstream.h"
 #include "giostream.h"
 
+#ifdef G_OS_UNIX
+#include <unistd.h>
+#endif
+#ifdef G_OS_WIN32
+#include <io.h>
+#endif
+
 #ifdef G_OS_UNIX
 #include "gunixsocketaddress.h"
 #endif
@@ -871,7 +872,7 @@ try_tcp (GDBusServer  *server,
               g_set_error (error,
                            G_IO_ERROR,
                            g_io_error_from_errno (errno),
-                           _("Error writing nonce file at `%s': %s"),
+                           _("Error writing nonce file at '%s': %s"),
                            server->nonce_file,
                            strerror (errno));
               goto out;
@@ -897,7 +898,8 @@ try_tcp (GDBusServer  *server,
 
  out:
   g_list_free_full (resolved_addresses, g_object_unref);
-  g_object_unref (resolver);
+  if (resolver)
+    g_object_unref (resolver);
   return ret;
 }
 
@@ -1040,7 +1042,7 @@ initable_init (GInitable     *initable,
       g_set_error (&last_error,
                    G_IO_ERROR,
                    G_IO_ERROR_INVALID_ARGUMENT,
-                   _("The string `%s' is not a valid D-Bus GUID"),
+                   _("The string '%s' is not a valid D-Bus GUID"),
                    server->guid);
       goto out;
     }
@@ -1080,7 +1082,7 @@ initable_init (GInitable     *initable,
             g_set_error (&this_error,
                          G_IO_ERROR,
                          G_IO_ERROR_INVALID_ARGUMENT,
-                         _("Cannot listen on unsupported transport `%s'"),
+                         _("Cannot listen on unsupported transport '%s'"),
                          transport_name);
 
           g_free (transport_name);
