@@ -2,12 +2,10 @@
  *
  * Copyright (C) 2008 Clemens N. Buss <cebuzz@gmail.com>
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <config.h>
@@ -31,11 +31,14 @@
 
 
 /**
- * GEmblem:
+ * SECTION:gemblem
+ * @short_description: An object for emblems
+ * @include: gio/gio.h
+ * @see_also: #GIcon, #GEmblemedIcon, #GLoadableIcon, #GThemedIcon
  *
- * `GEmblem` is an implementation of [iface@Gio.Icon] that supports
+ * #GEmblem is an implementation of #GIcon that supports
  * having an emblem, which is an icon with additional properties.
- * It can than be added to a [class@Gio.EmblemedIcon].
+ * It can than be added to a #GEmblemedIcon.
  *
  * Currently, only metainformation about the emblem's origin is
  * supported. More may be added in the future.
@@ -134,30 +137,20 @@ g_emblem_class_init (GEmblemClass *klass)
   gobject_class->set_property = g_emblem_set_property;
   gobject_class->get_property = g_emblem_get_property;
 
-  /**
-   * GEmblem:origin:
-   *
-   * The origin the emblem is derived from.
-   *
-   * Since: 2.18
-   */
   g_object_class_install_property (gobject_class,
                                    PROP_ORIGIN,
-                                   g_param_spec_enum ("origin", NULL, NULL,
+                                   g_param_spec_enum ("origin",
+                                                      P_("GEmblem's origin"),
+                                                      P_("Tells which origin the emblem is derived from"),
                                                       G_TYPE_EMBLEM_ORIGIN,
                                                       G_EMBLEM_ORIGIN_UNKNOWN,
                                                       G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GEmblem:icon:
-   *
-   * The actual icon of the emblem.
-   *
-   * Since: 2.18
-   */
   g_object_class_install_property (gobject_class,
                                    PROP_ICON,
-                                   g_param_spec_object ("icon", NULL, NULL,
+                                   g_param_spec_object ("icon",
+                                                      P_("The icon of the emblem"),
+                                                      P_("The actual icon of the emblem"),
                                                       G_TYPE_OBJECT,
                                                       G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -329,7 +322,7 @@ g_emblem_from_tokens (gchar  **tokens,
       g_set_error (error,
                    G_IO_ERROR,
                    G_IO_ERROR_INVALID_ARGUMENT,
-                   _("Can’t handle version %d of GEmblem encoding"),
+                   _("Can't handle version %d of GEmblem encoding"),
                    version);
       return NULL;
     }
