@@ -29,10 +29,6 @@
 #include <stdio.h>
 #include <locale.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include "gvdb/gvdb-builder.h"
 #include "strinfo.c"
 
@@ -1126,7 +1122,7 @@ parse_state_start_schema (ParseState  *state,
         {
           g_set_error (error, G_MARKUP_ERROR,
                        G_MARKUP_ERROR_INVALID_CONTENT,
-                       _("<schema id='%s'> extends not-yet-existing "
+                       _("<schema id='%s'> extends not yet existing "
                          "schema '%s'"), id, extends_name);
           return;
         }
@@ -1142,7 +1138,7 @@ parse_state_start_schema (ParseState  *state,
         {
           g_set_error (error, G_MARKUP_ERROR,
                        G_MARKUP_ERROR_INVALID_CONTENT,
-                       _("<schema id='%s'> is list of not-yet-existing "
+                       _("<schema id='%s'> is list of not yet existing "
                          "schema '%s'"), id, list_of);
           return;
         }
@@ -1728,7 +1724,8 @@ parse_gschema_files (gchar    **files,
 
       context = g_markup_parse_context_new (&parser,
                                             G_MARKUP_TREAT_CDATA_AS_TEXT |
-                                            G_MARKUP_PREFIX_ERROR_POSITION,
+                                            G_MARKUP_PREFIX_ERROR_POSITION |
+                                            G_MARKUP_IGNORE_QUALIFIED,
                                             &state, NULL);
 
 
