@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors: Lars Uebernickel <lars@uebernic.de>
  */
@@ -23,7 +25,6 @@
 #include "gdbusutils.h"
 #include "gicon.h"
 #include "gaction.h"
-#include "gioenumtypes.h"
 
 /**
  * SECTION:gnotification
@@ -43,29 +44,6 @@
  * not running, applications using #GNotification should be able to be
  * started as a D-Bus service, using #GApplication.
  *
- * In order for #GNotification to work, the application must have installed
- * a `.desktop` file. For example:
- * |[
- *  [Desktop Entry]
- *   Name=Test Application
- *   Comment=Description of what Test Application does
- *   Exec=gnome-test-application
- *   Icon=org.gnome.TestApplication
- *   Terminal=false
- *   Type=Application
- *   Categories=GNOME;GTK;TestApplication Category;
- *   StartupNotify=true
- *   DBusActivatable=true
- *   X-GNOME-UsesNotifications=true
- * ]|
- *
- * The `X-GNOME-UsesNotifications` key indicates to GNOME Control Center
- * that this application uses notifications, so it can be listed in the
- * Control Center’s ‘Notifications’ panel.
- *
- * The `.desktop` file must be named as `org.gnome.TestApplication.desktop`,
- * where `org.gnome.TestApplication` is the ID passed to g_application_new().
- *
  * User interaction with a notification (either the default action, or
  * buttons) must be associated with actions on the application (ie:
  * "app." actions).  It is not possible to route user interaction
@@ -96,8 +74,7 @@ struct _GNotification
   gchar *title;
   gchar *body;
   GIcon *icon;
-  GNotificationPriority priority;
-  gchar *category;
+  gboolean urgent;
   GPtrArray *buttons;
   gchar *default_action;
   GVariant *default_action_target;
@@ -110,7 +87,7 @@ typedef struct
   GVariant *target;
 } Button;
 
-G_DEFINE_TYPE (GNotification, g_notification, G_TYPE_OBJECT)
+G_DEFINE_TYPE (GNotification, g_notification, G_TYPE_OBJECT);
 
 static void
 button_free (gpointer data)
@@ -142,7 +119,6 @@ g_notification_finalize (GObject *object)
 
   g_free (notification->title);
   g_free (notification->body);
-  g_free (notification->category);
   g_free (notification->default_action);
   if (notification->default_action_target)
     g_variant_unref (notification->default_action_target);
@@ -239,7 +215,7 @@ g_notification_set_title (GNotification *notification,
  *
  * Gets the current body of @notification.
  *
- * Returns: (nullable): the body of @notification
+ * Returns: (allow-none): the body of @notification
  *
  * Since: 2.40
  */
@@ -254,7 +230,7 @@ g_notification_get_body (GNotification *notification)
 /**
  * g_notification_set_body:
  * @notification: a #GNotification
- * @body: (nullable): the new body for @notification, or %NULL
+ * @body: (allow-none): the new body for @notification, or %NULL
  *
  * Sets the body of @notification to @body.
  *
@@ -312,19 +288,19 @@ g_notification_set_icon (GNotification *notification,
 }
 
 /*< private >
- * g_notification_get_priority:
+ * g_notification_get_urgent:
  * @notification: a #GNotification
  *
- * Returns the priority of @notification
+ * Returns %TRUE if @notification is marked as urgent.
  *
- * Since: 2.42
+ * Since: 2.40
  */
-GNotificationPriority
-g_notification_get_priority (GNotification *notification)
+gboolean
+g_notification_get_urgent (GNotification *notification)
 {
-  g_return_val_if_fail (G_IS_NOTIFICATION (notification), G_NOTIFICATION_PRIORITY_NORMAL);
+  g_return_val_if_fail (G_IS_NOTIFICATION (notification), FALSE);
 
-  return notification->priority;
+  return notification->urgent;
 }
 
 /**
@@ -332,11 +308,9 @@ g_notification_get_priority (GNotification *notification)
  * @notification: a #GNotification
  * @urgent: %TRUE if @notification is urgent
  *
- * Deprecated in favor of g_notification_set_priority().
+ * Sets or unsets whether @notification is marked as urgent.
  *
  * Since: 2.40
- * Deprecated: 2.42: Since 2.42, this has been deprecated in favour of
- *    g_notification_set_priority().
  */
 void
 g_notification_set_urgent (GNotification *notification,
@@ -344,72 +318,7 @@ g_notification_set_urgent (GNotification *notification,
 {
   g_return_if_fail (G_IS_NOTIFICATION (notification));
 
-  notification->priority = urgent ?
-      G_NOTIFICATION_PRIORITY_URGENT :
-      G_NOTIFICATION_PRIORITY_NORMAL;
-}
-
-/*< private >
- * g_notification_get_category:
- * @notification: a #GNotification
- *
- * Gets the cateogry of @notification.
- *
- * This will be %NULL if no category is set.
- *
- * Returns: (nullable): the cateogry of @notification
- *
- * Since: 2.70
- */
-const gchar *
-g_notification_get_category (GNotification *notification)
-{
-  g_return_val_if_fail (G_IS_NOTIFICATION (notification), NULL);
-
-  return notification->category;
-}
-
-/**
- * g_notification_set_category:
- * @notification: a #GNotification
- * @category: (nullable): the category for @notification, or %NULL for no category
- *
- * Sets the type of @notification to @category. Categories have a main
- * type like `email`, `im` or `device` and can have a detail separated
- * by a `.`, e.g. `im.received` or `email.arrived`. Setting the category
- * helps the notification server to select proper feedback to the user.
- *
- * Standard categories are [listed in the specification](https://specifications.freedesktop.org/notification-spec/latest/ar01s06.html).
- *
- * Since: 2.70
- */
-void
-g_notification_set_category (GNotification *notification,
-                             const gchar   *category)
-{
-  g_return_if_fail (G_IS_NOTIFICATION (notification));
-  g_return_if_fail (category == NULL || *category != '\0');
-
-  g_free (notification->category);
-
-  notification->category = g_strdup (category);
-}
-
-/**
- * g_notification_set_priority:
- * @notification: a #GNotification
- * @priority: a #GNotificationPriority
- *
- * Sets the priority of @notification to @priority. See
- * #GNotificationPriority for possible values.
- */
-void
-g_notification_set_priority (GNotification         *notification,
-                             GNotificationPriority  priority)
-{
-  g_return_if_fail (G_IS_NOTIFICATION (notification));
-
-  notification->priority = priority;
+  notification->urgent = urgent;
 }
 
 /**
@@ -459,15 +368,15 @@ g_notification_add_button (GNotification *notification,
  * @notification: a #GNotification
  * @label: label of the button
  * @action: an action name
- * @target_format: (nullable): a #GVariant format string, or %NULL
- * @...: positional parameters, as determined by @target_format
+ * @target_format: (allow-none): a GVariant format string, or %NULL
+ * @...: positional parameters, as determined by @format_string
  *
  * Adds a button to @notification that activates @action when clicked.
  * @action must be an application-wide action (it must start with "app.").
  *
  * If @target_format is given, it is used to collect remaining
- * positional parameters into a #GVariant instance, similar to
- * g_variant_new(). @action will be activated with that #GVariant as its
+ * positional parameters into a GVariant instance, similar to
+ * g_variant_new(). @action will be activated with that GVariant as its
  * parameter.
  *
  * Since: 2.40
@@ -497,7 +406,7 @@ g_notification_add_button_with_target (GNotification *notification,
  * @notification: a #GNotification
  * @label: label of the button
  * @action: an action name
- * @target: (nullable): a #GVariant to use as @action's parameter, or %NULL
+ * @target: (allow-none): a GVariant to use as @action's parameter, or %NULL
  *
  * Adds a button to @notification that activates @action when clicked.
  * @action must be an application-wide action (it must start with "app.").
@@ -613,8 +522,8 @@ g_notification_get_button_with_action (GNotification *notification,
 /*< private >
  * g_notification_get_default_action:
  * @notification: a #GNotification
- * @action: (nullable): return location for the default action
- * @target: (nullable): return location for the target of the default action
+ * @action: (allow-none): return location for the default action
+ * @target: (allow-none): return location for the target of the default action
  *
  * Gets the action and target for the default action of @notification.
  *
@@ -687,16 +596,16 @@ g_notification_set_default_action (GNotification *notification,
  * g_notification_set_default_action_and_target: (skip)
  * @notification: a #GNotification
  * @action: an action name
- * @target_format: (nullable): a #GVariant format string, or %NULL
- * @...: positional parameters, as determined by @target_format
+ * @target_format: (allow-none): a GVariant format string, or %NULL
+ * @...: positional parameters, as determined by @format_string
  *
  * Sets the default action of @notification to @action. This action is
  * activated when the notification is clicked on. It must be an
  * application-wide action (it must start with "app.").
  *
  * If @target_format is given, it is used to collect remaining
- * positional parameters into a #GVariant instance, similar to
- * g_variant_new(). @action will be activated with that #GVariant as its
+ * positional parameters into a GVariant instance, similar to
+ * g_variant_new(). @action will be activated with that GVariant as its
  * parameter.
  *
  * When no default action is set, the application that the notification
@@ -727,12 +636,16 @@ g_notification_set_default_action_and_target (GNotification *notification,
  * g_notification_set_default_action_and_target_value: (rename-to g_notification_set_default_action_and_target)
  * @notification: a #GNotification
  * @action: an action name
- * @target: (nullable): a #GVariant to use as @action's parameter, or %NULL
+ * @target: (allow-none): a GVariant to use as @action's parameter, or %NULL
  *
  * Sets the default action of @notification to @action. This action is
  * activated when the notification is clicked on. It must be an
  * application-wide action (start with "app.").
  *
+ * If @target_format is given, it is used to collect remaining
+ * positional parameters into a GVariant instance, similar to
+ * g_variant_new().
+ *
  * If @target is non-%NULL, @action will be activated with @target as
  * its parameter.
  *
@@ -780,26 +693,10 @@ g_notification_serialize_button (Button *button)
   return g_variant_builder_end (&builder);
 }
 
-static GVariant *
-g_notification_get_priority_nick (GNotification *notification)
-{
-  GEnumClass *enum_class;
-  GEnumValue *value;
-  GVariant *nick;
-
-  enum_class = g_type_class_ref (G_TYPE_NOTIFICATION_PRIORITY);
-  value = g_enum_get_value (enum_class, g_notification_get_priority (notification));
-  g_assert (value != NULL);
-  nick = g_variant_new_string (value->value_nick);
-  g_type_class_unref (enum_class);
-
-  return nick;
-}
-
 /*< private >
  * g_notification_serialize:
  *
- * Serializes @notification into a floating variant of type a{sv}.
+ * Serializes @notification into an floating variant of type a{sv}.
  *
  * Returns: the serialized @notification as a floating variant.
  */
@@ -827,7 +724,7 @@ g_notification_serialize (GNotification *notification)
         }
     }
 
-  g_variant_builder_add (&builder, "{sv}", "priority", g_notification_get_priority_nick (notification));
+  g_variant_builder_add (&builder, "{sv}", "urgent", g_variant_new_boolean (notification->urgent));
 
   if (notification->default_action)
     {
