@@ -26,6 +26,7 @@
  * SECTION:gsocketservice
  * @title: GSocketService
  * @short_description: Make it easy to implement a network service
+ * @include: gio/gio.h
  * @see_also: #GThreadedSocketService, #GSocketListener.
  *
  * A #GSocketService is an object that represents a service that
@@ -65,13 +66,6 @@
 #include "gsocketlistener.h"
 #include "gsocketconnection.h"
 
-
-static guint g_socket_service_incoming_signal;
-
-G_DEFINE_TYPE (GSocketService, g_socket_service, G_TYPE_SOCKET_LISTENER);
-
-G_LOCK_DEFINE_STATIC(active);
-
 struct _GSocketServicePrivate
 {
   GCancellable *cancellable;
@@ -79,6 +73,12 @@ struct _GSocketServicePrivate
   guint outstanding_accept : 1;
 };
 
+static guint g_socket_service_incoming_signal;
+
+G_LOCK_DEFINE_STATIC(active);
+
+G_DEFINE_TYPE_WITH_PRIVATE (GSocketService, g_socket_service, G_TYPE_SOCKET_LISTENER)
+
 static void g_socket_service_ready (GObject      *object,
 				    GAsyncResult *result,
 				    gpointer      user_data);
@@ -94,9 +94,7 @@ g_socket_service_real_incoming (GSocketService    *service,
 static void
 g_socket_service_init (GSocketService *service)
 {
-  service->priv = G_TYPE_INSTANCE_GET_PRIVATE (service,
-					       G_TYPE_SOCKET_SERVICE,
-					       GSocketServicePrivate);
+  service->priv = g_socket_service_get_instance_private (service);
   service->priv->cancellable = g_cancellable_new ();
   service->priv->active = TRUE;
 }
@@ -244,8 +242,6 @@ g_socket_service_class_init (GSocketServiceClass *class)
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
   GSocketListenerClass *listener_class = G_SOCKET_LISTENER_CLASS (class);
 
-  g_type_class_add_private (class, sizeof (GSocketServicePrivate));
-
   gobject_class->finalize = g_socket_service_finalize;
   listener_class->changed = g_socket_service_changed;
   class->incoming = g_socket_service_real_incoming;
