@@ -31,6 +31,7 @@
 #include "gioerror.h"
 #include "gsimpleasyncresult.h"
 #include "gwin32inputstream.h"
+#include "giowin32-priv.h"
 #include "gcancellable.h"
 #include "gasynchelper.h"
 #include "glibintl.h"
@@ -56,13 +57,14 @@ enum {
   PROP_CLOSE_HANDLE
 };
 
-G_DEFINE_TYPE (GWin32InputStream, g_win32_input_stream, G_TYPE_INPUT_STREAM);
-
 struct _GWin32InputStreamPrivate {
   HANDLE handle;
   gboolean close_handle;
+  gint fd;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (GWin32InputStream, g_win32_input_stream, G_TYPE_INPUT_STREAM)
+
 static void     g_win32_input_stream_set_property (GObject              *object,
 						   guint                 prop_id,
 						   const GValue         *value,
@@ -80,23 +82,14 @@ static gboolean g_win32_input_stream_close        (GInputStream         *stream,
 						   GCancellable         *cancellable,
 						   GError              **error);
 
-static void
-g_win32_input_stream_finalize (GObject *object)
-{
-  G_OBJECT_CLASS (g_win32_input_stream_parent_class)->finalize (object);
-}
-
 static void
 g_win32_input_stream_class_init (GWin32InputStreamClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GInputStreamClass *stream_class = G_INPUT_STREAM_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (GWin32InputStreamPrivate));
-
   gobject_class->get_property = g_win32_input_stream_get_property;
   gobject_class->set_property = g_win32_input_stream_set_property;
-  gobject_class->finalize = g_win32_input_stream_finalize;
 
   stream_class->read_fn = g_win32_input_stream_read;
   stream_class->close_fn = g_win32_input_stream_close;
@@ -181,12 +174,10 @@ g_win32_input_stream_get_property (GObject    *object,
 static void
 g_win32_input_stream_init (GWin32InputStream *win32_stream)
 {
-  win32_stream->priv = G_TYPE_INSTANCE_GET_PRIVATE (win32_stream,
-						    G_TYPE_WIN32_INPUT_STREAM,
-						    GWin32InputStreamPrivate);
-
+  win32_stream->priv = g_win32_input_stream_get_instance_private (win32_stream);
   win32_stream->priv->handle = NULL;
   win32_stream->priv->close_handle = TRUE;
+  win32_stream->priv->fd = -1;
 }
 
 /**
@@ -376,19 +367,44 @@ g_win32_input_stream_close (GInputStream  *stream,
   if (!win32_stream->priv->close_handle)
     return TRUE;
 
-  res = CloseHandle (win32_stream->priv->handle);
-  if (!res)
+  if (win32_stream->priv->fd != -1)
     {
-      int errsv = GetLastError ();
-      gchar *emsg = g_win32_error_message (errsv);
+      if (close (win32_stream->priv->fd) < 0)
+	{
+	  g_set_error_literal (error, G_IO_ERROR,
+			       g_io_error_from_errno (errno),
+			       g_strerror (errno));
+	  return FALSE;
+	}
+    }
+  else
+    {
+      res = CloseHandle (win32_stream->priv->handle);
+      if (!res)
+	{
+	  int errsv = GetLastError ();
+	  gchar *emsg = g_win32_error_message (errsv);
 
-      g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_win32_error (errsv),
-		   _("Error closing handle: %s"),
-		   emsg);
-      g_free (emsg);
-      return FALSE;
+	  g_set_error (error, G_IO_ERROR,
+		       g_io_error_from_win32_error (errsv),
+		       _("Error closing handle: %s"),
+		       emsg);
+	  g_free (emsg);
+	  return FALSE;
+	}
     }
 
   return TRUE;
 }
+
+GInputStream *
+g_win32_input_stream_new_from_fd (gint      fd,
+				  gboolean  close_fd)
+{
+  GWin32InputStream *win32_stream;
+
+  win32_stream = G_WIN32_INPUT_STREAM (g_win32_input_stream_new ((HANDLE) _get_osfhandle (fd), close_fd));
+  win32_stream->priv->fd = fd;
+
+  return (GInputStream*)win32_stream;
+}
