@@ -1,6 +1,18 @@
 #include <gio/gio.h>
 #include <string.h>
 
+#define g_assert_content_type_equals(s1, s2) 			\
+  do { 								\
+    const char *__s1 = (s1), *__s2 = (s2); 			\
+    if (g_content_type_equals (__s1, __s2)) ;		 	\
+    else 							\
+      g_assertion_message_cmpstr (G_LOG_DOMAIN, 		\
+                                  __FILE__, __LINE__, 		\
+                                  G_STRFUNC, 			\
+                                  #s1 " == " #s2, 		\
+                                  __s1, " == ", __s2); 		\
+  } while (0)
+
 static void
 test_guess (void)
 {
@@ -15,41 +27,41 @@ test_guess (void)
 
   res = g_content_type_guess ("/etc/", NULL, 0, &uncertain);
   expected = g_content_type_from_mime_type ("inode/directory");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   g_assert (uncertain);
   g_free (res);
   g_free (expected);
 
   res = g_content_type_guess ("foo.txt", NULL, 0, &uncertain);
   expected = g_content_type_from_mime_type ("text/plain");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   g_free (res);
   g_free (expected);
 
   res = g_content_type_guess ("foo.desktop", data, sizeof (data) - 1, &uncertain);
   expected = g_content_type_from_mime_type ("application/x-desktop");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   g_assert (!uncertain);
   g_free (res);
   g_free (expected);
 
   res = g_content_type_guess ("foo.txt", data, sizeof (data) - 1, &uncertain);
   expected = g_content_type_from_mime_type ("text/plain");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   g_assert (!uncertain);
   g_free (res);
   g_free (expected);
 
   res = g_content_type_guess ("foo", data, sizeof (data) - 1, &uncertain);
   expected = g_content_type_from_mime_type ("text/plain");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   g_assert (!uncertain);
   g_free (res);
   g_free (expected);
 
   res = g_content_type_guess (NULL, data, sizeof (data) - 1, &uncertain);
   expected = g_content_type_from_mime_type ("application/x-desktop");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   g_assert (!uncertain);
   g_free (res);
   g_free (expected);
@@ -58,29 +70,37 @@ test_guess (void)
    * but looks like text so it can't be Powerpoint */
   res = g_content_type_guess ("test.pot", (guchar *)"ABC abc", 7, &uncertain);
   expected = g_content_type_from_mime_type ("text/x-gettext-translation-template");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   g_assert (!uncertain);
   g_free (res);
   g_free (expected);
 
   res = g_content_type_guess ("test.pot", (guchar *)"msgid \"", 7, &uncertain);
   expected = g_content_type_from_mime_type ("text/x-gettext-translation-template");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   g_assert (!uncertain);
   g_free (res);
   g_free (expected);
 
   res = g_content_type_guess ("test.pot", (guchar *)"\xCF\xD0\xE0\x11", 4, &uncertain);
   expected = g_content_type_from_mime_type ("application/vnd.ms-powerpoint");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
   /* we cannot reliably detect binary powerpoint files as long as there is no
-   * defined MIME magic, so do not check uncertain here */
+   * defined MIME magic, so do not check uncertain here
+   */
   g_free (res);
   g_free (expected);
 
   res = g_content_type_guess ("test.otf", (guchar *)"OTTO", 4, &uncertain);
   expected = g_content_type_from_mime_type ("application/x-font-otf");
-  g_assert (g_content_type_equals (expected, res));
+  g_assert_content_type_equals (expected, res);
+  g_assert (!uncertain);
+  g_free (res);
+  g_free (expected);
+
+  res = g_content_type_guess (NULL, (guchar *)"%!PS-Adobe-2.0 EPSF-1.2", 23, &uncertain);
+  expected = g_content_type_from_mime_type ("image/x-eps");
+  g_assert_content_type_equals (expected, res);
   g_assert (!uncertain);
   g_free (res);
   g_free (expected);
@@ -198,6 +218,29 @@ test_icon (void)
   g_free (type);
 }
 
+static void
+test_tree (void)
+{
+  const gchar *tests[] = {
+    "x-content/image-dcf",
+    "x-content/unix-software",
+    "x-content/win32-software"
+  };
+  const gchar *path;
+  GFile *file;
+  gchar **types;
+  gint i;
+
+  for (i = 0; i < G_N_ELEMENTS (tests); i++)
+    {
+      path = g_test_get_filename (G_TEST_DIST, tests[i], NULL);
+      file = g_file_new_for_path (path);
+      types = g_content_type_guess_for_tree (file);
+      g_assert_content_type_equals (types[0], tests[i]);
+      g_strfreev (types);
+      g_object_unref (file);
+   }
+}
 
 int
 main (int argc, char *argv[])
@@ -211,6 +254,7 @@ main (int argc, char *argv[])
   g_test_add_func ("/contenttype/executable", test_executable);
   g_test_add_func ("/contenttype/description", test_description);
   g_test_add_func ("/contenttype/icon", test_icon);
+  g_test_add_func ("/contenttype/tree", test_tree);
 
   return g_test_run ();
 }
