@@ -118,7 +118,6 @@ struct _GRealArray
  * @a: a #GArray.
  * @t: the type of the elements.
  * @i: the index of the element to return.
- * @Returns: the element of the #GArray at the index given by @i.
  *
  * Returns the element of a #GArray at the given index. The return
  * value is cast to the given type.
@@ -132,6 +131,8 @@ struct _GRealArray
  *   event = &amp;g_array_index (events, EDayViewEvent, 3);
  *  </programlisting>
  * </example>
+ *
+ * Returns: the element of the #GArray at the index given by @i.
  **/
 
 #define g_array_elt_len(array,i) ((array)->elt_size * (i))
@@ -154,9 +155,10 @@ static void  g_array_maybe_expand (GRealArray *array,
  * @clear_: %TRUE if #GArray elements should be automatically cleared
  *          to 0 when they are allocated.
  * @element_size: the size of each element in bytes.
- * @Returns: the new #GArray.
  *
  * Creates a new #GArray with a reference count of 1.
+ *
+ * Returns: the new #GArray.
  **/
 GArray*
 g_array_new (gboolean zero_terminated,
@@ -176,12 +178,13 @@ g_array_new (gboolean zero_terminated,
  *          allocation.
  * @element_size: size of each element in the array.
  * @reserved_size: number of elements preallocated.
- * @Returns: the new #GArray.
  *
  * Creates a new #GArray with @reserved_size elements preallocated and
  * a reference count of 1. This avoids frequent reallocation, if you
  * are going to add many elements to the array. Note however that the
  * size of the array is still 0.
+ *
+ * Returns: the new #GArray.
  **/
 GArray* g_array_sized_new (gboolean zero_terminated,
 			   gboolean clear,
@@ -315,8 +318,6 @@ g_array_get_element_size (GArray *array)
  * g_array_free:
  * @array: a #GArray.
  * @free_segment: if %TRUE the actual element data is freed as well.
- * @Returns: the element data if @free_segment is %FALSE, otherwise
- *           %NULL.  The element data should be freed using g_free().
  *
  * Frees the memory allocated for the #GArray. If @free_segment is
  * %TRUE it frees the memory block holding the elements as well and
@@ -328,6 +329,9 @@ g_array_get_element_size (GArray *array)
  *
  * <note><para>If array elements contain dynamically-allocated memory,
  * they should be freed separately.</para></note>
+ *
+ * Returns: the element data if @free_segment is %FALSE, otherwise
+ *          %NULL.  The element data should be freed using g_free().
  **/
 gchar*
 g_array_free (GArray   *farray,
@@ -388,15 +392,15 @@ array_free (GRealArray     *array,
  * @array: a #GArray.
  * @data: a pointer to the elements to append to the end of the array.
  * @len: the number of elements to append.
- * @Returns: the #GArray.
  *
  * Adds @len elements onto the end of the array.
+ *
+ * Returns: the #GArray.
  **/
 /**
  * g_array_append_val:
  * @a: a #GArray.
  * @v: the value to append to the #GArray.
- * @Returns: the #GArray.
  *
  * Adds the value on to the end of the array. The array will grow in
  * size automatically if necessary.
@@ -404,6 +408,8 @@ array_free (GRealArray     *array,
  * <note><para>g_array_append_val() is a macro which uses a reference
  * to the value parameter @v. This means that you cannot use it with
  * literal values such as "27". You must use variables.</para></note>
+ *
+ * Returns: the #GArray.
  **/
 GArray*
 g_array_append_vals (GArray       *farray,
@@ -432,19 +438,19 @@ g_array_append_vals (GArray       *farray,
  * @data: a pointer to the elements to prepend to the start of the
  *        array.
  * @len: the number of elements to prepend.
- * @Returns: the #GArray.
  *
  * Adds @len elements onto the start of the array.
  *
  * This operation is slower than g_array_append_vals() since the
  * existing elements in the array have to be moved to make space for
  * the new elements.
+ *
+ * Returns: the #GArray.
  **/
 /**
  * g_array_prepend_val:
  * @a: a #GArray.
  * @v: the value to prepend to the #GArray.
- * @Returns: the #GArray.
  *
  * Adds the value on to the start of the array. The array will grow in
  * size automatically if necessary.
@@ -456,6 +462,8 @@ g_array_append_vals (GArray       *farray,
  * <note><para>g_array_prepend_val() is a macro which uses a reference
  * to the value parameter @v. This means that you cannot use it with
  * literal values such as "27". You must use variables.</para></note>
+ *
+ * Returns: the #GArray.
  **/
 GArray*
 g_array_prepend_vals (GArray        *farray,
@@ -468,8 +476,8 @@ g_array_prepend_vals (GArray        *farray,
 
   g_array_maybe_expand (array, len);
 
-  g_memmove (g_array_elt_pos (array, len), g_array_elt_pos (array, 0), 
-	     g_array_elt_len (array, array->len));
+  memmove (g_array_elt_pos (array, len), g_array_elt_pos (array, 0),
+           g_array_elt_len (array, array->len));
 
   memcpy (g_array_elt_pos (array, 0), data, g_array_elt_len (array, len));
 
@@ -486,22 +494,24 @@ g_array_prepend_vals (GArray        *farray,
  * @index_: the index to place the elements at.
  * @data: a pointer to the elements to insert.
  * @len: the number of elements to insert.
- * @Returns: the #GArray.
  *
  * Inserts @len elements into a #GArray at the given index.
+ *
+ * Returns: the #GArray.
  **/
 /**
  * g_array_insert_val:
  * @a: a #GArray.
  * @i: the index to place the element at.
  * @v: the value to insert into the array.
- * @Returns: the #GArray.
  *
  * Inserts an element into an array at the given index.
  *
  * <note><para>g_array_insert_val() is a macro which uses a reference
  * to the value parameter @v. This means that you cannot use it with
  * literal values such as "27". You must use variables.</para></note>
+ *
+ * Returns: the #GArray.
  **/
 GArray*
 g_array_insert_vals (GArray        *farray,
@@ -515,9 +525,9 @@ g_array_insert_vals (GArray        *farray,
 
   g_array_maybe_expand (array, len);
 
-  g_memmove (g_array_elt_pos (array, len + index_), 
-	     g_array_elt_pos (array, index_), 
-	     g_array_elt_len (array, array->len - index_));
+  memmove (g_array_elt_pos (array, len + index_),
+           g_array_elt_pos (array, index_),
+           g_array_elt_len (array, array->len - index_));
 
   memcpy (g_array_elt_pos (array, index_), data, g_array_elt_len (array, len));
 
@@ -532,10 +542,11 @@ g_array_insert_vals (GArray        *farray,
  * g_array_set_size:
  * @array: a #GArray.
  * @length: the new size of the #GArray.
- * @Returns: the #GArray.
  *
  * Sets the size of the array, expanding it if necessary. If the array
  * was created with @clear_ set to %TRUE, the new elements are set to 0.
+ *
+ * Returns: the #GArray.
  **/
 GArray*
 g_array_set_size (GArray *farray,
@@ -566,10 +577,11 @@ g_array_set_size (GArray *farray,
  * g_array_remove_index:
  * @array: a #GArray.
  * @index_: the index of the element to remove.
- * @Returns: the #GArray.
  *
  * Removes the element at the given index from a #GArray. The following
  * elements are moved down one place.
+ *
+ * Returns: the #GArray.
  **/
 GArray*
 g_array_remove_index (GArray *farray,
@@ -585,9 +597,9 @@ g_array_remove_index (GArray *farray,
     array->clear_func (g_array_elt_pos (array, index_));
 
   if (index_ != array->len - 1)
-    g_memmove (g_array_elt_pos (array, index_),
-               g_array_elt_pos (array, index_ + 1),
-               g_array_elt_len (array, array->len - index_ - 1));
+    memmove (g_array_elt_pos (array, index_),
+             g_array_elt_pos (array, index_ + 1),
+             g_array_elt_len (array, array->len - index_ - 1));
 
   array->len -= 1;
 
@@ -603,12 +615,13 @@ g_array_remove_index (GArray *farray,
  * g_array_remove_index_fast:
  * @array: a @GArray.
  * @index_: the index of the element to remove.
- * @Returns: the #GArray.
  *
  * Removes the element at the given index from a #GArray. The last
  * element in the array is used to fill in the space, so this function
  * does not preserve the order of the #GArray. But it is faster than
  * g_array_remove_index().
+ *
+ * Returns: the #GArray.
  **/
 GArray*
 g_array_remove_index_fast (GArray *farray,
@@ -643,11 +656,12 @@ g_array_remove_index_fast (GArray *farray,
  * @array: a @GArray.
  * @index_: the index of the first element to remove.
  * @length: the number of elements to remove.
- * @Returns: the #GArray.
  *
  * Removes the given number of elements starting at the given index
  * from a #GArray.  The following elements are moved to close the gap.
  *
+ * Returns: the #GArray.
+ *
  * Since: 2.4
  **/
 GArray*
@@ -670,9 +684,9 @@ g_array_remove_range (GArray *farray,
     }
 
   if (index_ + length != array->len)
-    g_memmove (g_array_elt_pos (array, index_),
-               g_array_elt_pos (array, index_ + length),
-               (array->len - (index_ + length)) * array->elt_size);
+    memmove (g_array_elt_pos (array, index_),
+             g_array_elt_pos (array, index_ + length),
+             (array->len - (index_ + length)) * array->elt_size);
 
   array->len -= length;
   if (G_UNLIKELY (g_mem_gc_friendly))
@@ -848,9 +862,14 @@ struct _GRealPtrArray
  * g_ptr_array_index:
  * @array: a #GPtrArray.
  * @index_: the index of the pointer to return.
- * @Returns: the pointer at the given index.
  *
  * Returns the pointer at the given index of the pointer array.
+ *
+ * <note><para>
+ * This does not perform bounds checking on the given @index_, so you are
+ * responsible for checking it against the array length.</para></note>
+ *
+ * Returns: the pointer at the given index.
  **/
 
 static void g_ptr_array_maybe_expand (GRealPtrArray *array,
@@ -858,9 +877,10 @@ static void g_ptr_array_maybe_expand (GRealPtrArray *array,
 
 /**
  * g_ptr_array_new:
- * @Returns: the new #GPtrArray.
  *
  * Creates a new #GPtrArray with a reference count of 1.
+ *
+ * Returns: the new #GPtrArray.
  **/
 GPtrArray*
 g_ptr_array_new (void)
@@ -871,12 +891,13 @@ g_ptr_array_new (void)
 /**
  * g_ptr_array_sized_new:
  * @reserved_size: number of pointers preallocated.
- * @Returns: the new #GPtrArray.
  *
  * Creates a new #GPtrArray with @reserved_size pointers preallocated
  * and a reference count of 1. This avoids frequent reallocation, if
  * you are going to add many pointers to the array. Note however that
  * the size of the array is still 0.
+ *
+ * Returns: the new #GPtrArray.
  **/
 GPtrArray*  
 g_ptr_array_sized_new (guint reserved_size)
@@ -1018,8 +1039,6 @@ g_ptr_array_unref (GPtrArray *array)
  * g_ptr_array_free:
  * @array: a #GPtrArray.
  * @free_seg: if %TRUE the actual pointer array is freed as well.
- * @Returns: the pointer array if @free_seg is %FALSE, otherwise %NULL.
- *           The pointer array should be freed using g_free().
  *
  * Frees the memory allocated for the #GPtrArray. If @free_seg is %TRUE
  * it frees the memory block holding the elements as well. Pass %FALSE
@@ -1031,6 +1050,9 @@ g_ptr_array_unref (GPtrArray *array)
  * <note><para>If array contents point to dynamically-allocated
  * memory, they should be freed separately if @free_seg is %TRUE and no
  * #GDestroyNotify function has been set for @array.</para></note>
+ *
+ * Returns: the pointer array if @free_seg is %FALSE, otherwise %NULL.
+ *          The pointer array should be freed using g_free().
  **/
 gpointer*
 g_ptr_array_free (GPtrArray *farray,
@@ -1138,12 +1160,13 @@ g_ptr_array_set_size  (GPtrArray *farray,
  * g_ptr_array_remove_index:
  * @array: a #GPtrArray.
  * @index_: the index of the pointer to remove.
- * @Returns: the pointer which was removed.
  *
  * Removes the pointer at the given index from the pointer array. The
  * following elements are moved down one place. If @array has a
  * non-%NULL #GDestroyNotify function it is called for the removed
  * element.
+ *
+ * Returns: the pointer which was removed.
  **/
 gpointer
 g_ptr_array_remove_index (GPtrArray *farray,
@@ -1162,8 +1185,8 @@ g_ptr_array_remove_index (GPtrArray *farray,
     array->element_free_func (array->pdata[index_]);
 
   if (index_ != array->len - 1)
-    g_memmove (array->pdata + index_, array->pdata + index_ + 1, 
-               sizeof (gpointer) * (array->len - index_ - 1));
+    memmove (array->pdata + index_, array->pdata + index_ + 1,
+             sizeof (gpointer) * (array->len - index_ - 1));
   
   array->len -= 1;
 
@@ -1177,13 +1200,14 @@ g_ptr_array_remove_index (GPtrArray *farray,
  * g_ptr_array_remove_index_fast:
  * @array: a #GPtrArray.
  * @index_: the index of the pointer to remove.
- * @Returns: the pointer which was removed.
  *
  * Removes the pointer at the given index from the pointer array. The
  * last element in the array is used to fill in the space, so this
  * function does not preserve the order of the array. But it is faster
  * than g_ptr_array_remove_index(). If @array has a non-%NULL
  * #GDestroyNotify function it is called for the removed element.
+ *
+ * Returns: the pointer which was removed.
  **/
 gpointer
 g_ptr_array_remove_index_fast (GPtrArray *farray,
@@ -1214,18 +1238,20 @@ g_ptr_array_remove_index_fast (GPtrArray *farray,
 
 /**
  * g_ptr_array_remove_range:
- * @array: a @GPtrArray.
- * @index_: the index of the first pointer to remove.
- * @length: the number of pointers to remove.
+ * @array: a @GPtrArray
+ * @index_: the index of the first pointer to remove
+ * @length: the number of pointers to remove
  *
  * Removes the given number of pointers starting at the given index
  * from a #GPtrArray.  The following elements are moved to close the
  * gap. If @array has a non-%NULL #GDestroyNotify function it is called
  * for the removed elements.
  *
+ * Returns: the @array
+ *
  * Since: 2.4
  **/
-void
+GPtrArray *
 g_ptr_array_remove_range (GPtrArray *farray,
                           guint      index_,
                           guint      length)
@@ -1233,9 +1259,9 @@ g_ptr_array_remove_range (GPtrArray *farray,
   GRealPtrArray* array = (GRealPtrArray*) farray;
   guint n;
 
-  g_return_if_fail (array);
-  g_return_if_fail (index_ < array->len);
-  g_return_if_fail (index_ + length <= array->len);
+  g_return_val_if_fail (array != NULL, NULL);
+  g_return_val_if_fail (index_ < array->len, NULL);
+  g_return_val_if_fail (index_ + length <= array->len, NULL);
 
   if (array->element_free_func != NULL)
     {
@@ -1245,9 +1271,9 @@ g_ptr_array_remove_range (GPtrArray *farray,
 
   if (index_ + length != array->len)
     {
-      g_memmove (&array->pdata[index_],
-                 &array->pdata[index_ + length], 
-                 (array->len - (index_ + length)) * sizeof (gpointer));
+      memmove (&array->pdata[index_],
+               &array->pdata[index_ + length],
+               (array->len - (index_ + length)) * sizeof (gpointer));
     }
 
   array->len -= length;
@@ -1257,14 +1283,14 @@ g_ptr_array_remove_range (GPtrArray *farray,
       for (i = 0; i < length; i++)
         array->pdata[array->len + i] = NULL;
     }
+
+  return farray;
 }
 
 /**
  * g_ptr_array_remove:
  * @array: a #GPtrArray.
  * @data: the pointer to remove.
- * @Returns: %TRUE if the pointer is removed. %FALSE if the pointer is
- *           not found in the array.
  *
  * Removes the first occurrence of the given pointer from the pointer
  * array. The following elements are moved down one place. If @array
@@ -1273,6 +1299,9 @@ g_ptr_array_remove_range (GPtrArray *farray,
  *
  * It returns %TRUE if the pointer was removed, or %FALSE if the
  * pointer was not found.
+ *
+ * Returns: %TRUE if the pointer is removed. %FALSE if the pointer is
+ *          not found in the array.
  **/
 gboolean
 g_ptr_array_remove (GPtrArray *farray,
@@ -1299,7 +1328,6 @@ g_ptr_array_remove (GPtrArray *farray,
  * g_ptr_array_remove_fast:
  * @array: a #GPtrArray.
  * @data: the pointer to remove.
- * @Returns: %TRUE if the pointer was found in the array.
  *
  * Removes the first occurrence of the given pointer from the pointer
  * array. The last element in the array is used to fill in the space,
@@ -1309,6 +1337,8 @@ g_ptr_array_remove (GPtrArray *farray,
  *
  * It returns %TRUE if the pointer was removed, or %FALSE if the
  * pointer was not found.
+ *
+ * Returns: %TRUE if the pointer was found in the array.
  **/
 gboolean
 g_ptr_array_remove_fast (GPtrArray *farray,
@@ -1352,6 +1382,42 @@ g_ptr_array_add (GPtrArray *farray,
   array->pdata[array->len++] = data;
 }
 
+/**
+ * g_ptr_array_insert:
+ * @array: a #GPtrArray.
+ * @index_: the index to place the new element at, or -1 to append.
+ * @data: the pointer to add.
+ *
+ * Inserts an element into the pointer array at the given index. The 
+ * array will grow in size automatically if necessary.
+ *
+ * Since: 2.40
+ **/
+void
+g_ptr_array_insert (GPtrArray *farray,
+                    gint       index_,
+                    gpointer   data)
+{
+  GRealPtrArray* array = (GRealPtrArray*) farray;
+
+  g_return_if_fail (array);
+  g_return_if_fail (index_ >= -1);
+  g_return_if_fail (index_ <= (gint)array->len);
+
+  g_ptr_array_maybe_expand (array, 1);
+
+  if (index_ < 0)
+    index_ = array->len;
+
+  if (index_ < array->len)
+    memmove (&(array->pdata[index_ + 1]),
+             &(array->pdata[index_]),
+             (array->len - index_) * sizeof (gpointer));
+
+  array->len++;
+  array->pdata[index_] = data;
+}
+
 /**
  * g_ptr_array_sort:
  * @array: a #GPtrArray.
@@ -1487,9 +1553,10 @@ g_ptr_array_foreach (GPtrArray *array,
 
 /**
  * g_byte_array_new:
- * @Returns: the new #GByteArray.
  *
  * Creates a new #GByteArray with a reference count of 1.
+ *
+ * Returns: (transfer full): the new #GByteArray.
  **/
 GByteArray* g_byte_array_new (void)
 {
@@ -1498,7 +1565,7 @@ GByteArray* g_byte_array_new (void)
 
 /**
  * g_byte_array_new_take:
- * @data: (array length=len): byte data for the array
+ * @data: (transfer full) (array length=len): byte data for the array
  * @len: length of @data
  *
  * Create byte array containing the data. The data will be owned by the array
@@ -1529,12 +1596,13 @@ g_byte_array_new_take (guint8 *data,
 /**
  * g_byte_array_sized_new:
  * @reserved_size: number of bytes preallocated.
- * @Returns: the new #GByteArray.
  *
  * Creates a new #GByteArray with @reserved_size bytes preallocated.
  * This avoids frequent reallocation, if you are going to add many
  * bytes to the array. Note however that the size of the array is still
  * 0.
+ *
+ * Returns: the new #GByteArray.
  **/
 GByteArray* g_byte_array_sized_new (guint reserved_size)
 {
@@ -1545,13 +1613,14 @@ GByteArray* g_byte_array_sized_new (guint reserved_size)
  * g_byte_array_free:
  * @array: a #GByteArray.
  * @free_segment: if %TRUE the actual byte data is freed as well.
- * @Returns: the element data if @free_segment is %FALSE, otherwise
- *           %NULL.  The element data should be freed using g_free().
  *
  * Frees the memory allocated by the #GByteArray. If @free_segment is
  * %TRUE it frees the actual byte data. If the reference count of
  * @array is greater than one, the #GByteArray wrapper is preserved but
  * the size of @array will be set to zero.
+ *
+ * Returns: the element data if @free_segment is %FALSE, otherwise
+ *          %NULL.  The element data should be freed using g_free().
  **/
 guint8*	    g_byte_array_free     (GByteArray *array,
 			           gboolean    free_segment)
@@ -1627,10 +1696,11 @@ g_byte_array_unref (GByteArray *array)
  * @array: a #GByteArray.
  * @data: the byte data to be added.
  * @len: the number of bytes to add.
- * @Returns: the #GByteArray.
  *
  * Adds the given bytes to the end of the #GByteArray. The array will
  * grow in size automatically if necessary.
+ *
+ * Returns: the #GByteArray.
  **/
 GByteArray* g_byte_array_append   (GByteArray   *array,
 				   const guint8 *data,
@@ -1646,10 +1716,11 @@ GByteArray* g_byte_array_append   (GByteArray   *array,
  * @array: a #GByteArray.
  * @data: the byte data to be added.
  * @len: the number of bytes to add.
- * @Returns: the #GByteArray.
  *
  * Adds the given data to the start of the #GByteArray. The array will
  * grow in size automatically if necessary.
+ *
+ * Returns: the #GByteArray.
  **/
 GByteArray* g_byte_array_prepend  (GByteArray   *array,
 				   const guint8 *data,
@@ -1664,9 +1735,10 @@ GByteArray* g_byte_array_prepend  (GByteArray   *array,
  * g_byte_array_set_size:
  * @array: a #GByteArray.
  * @length: the new size of the #GByteArray.
- * @Returns: the #GByteArray.
  *
  * Sets the size of the #GByteArray, expanding it if necessary.
+ *
+ * Returns: the #GByteArray.
  **/
 GByteArray* g_byte_array_set_size (GByteArray *array,
 				   guint       length)
@@ -1680,10 +1752,11 @@ GByteArray* g_byte_array_set_size (GByteArray *array,
  * g_byte_array_remove_index:
  * @array: a #GByteArray.
  * @index_: the index of the byte to remove.
- * @Returns: the #GByteArray.
  *
  * Removes the byte at the given index from a #GByteArray. The
  * following bytes are moved down one place.
+ *
+ * Returns: the #GByteArray.
  **/
 GByteArray* g_byte_array_remove_index (GByteArray *array,
 				       guint       index_)
@@ -1697,12 +1770,13 @@ GByteArray* g_byte_array_remove_index (GByteArray *array,
  * g_byte_array_remove_index_fast:
  * @array: a #GByteArray.
  * @index_: the index of the byte to remove.
- * @Returns: the #GByteArray.
  *
  * Removes the byte at the given index from a #GByteArray. The last
  * element in the array is used to fill in the space, so this function
  * does not preserve the order of the #GByteArray. But it is faster
  * than g_byte_array_remove_index().
+ *
+ * Returns: the #GByteArray.
  **/
 GByteArray* g_byte_array_remove_index_fast (GByteArray *array,
 					    guint       index_)
@@ -1717,11 +1791,12 @@ GByteArray* g_byte_array_remove_index_fast (GByteArray *array,
  * @array: a @GByteArray.
  * @index_: the index of the first byte to remove.
  * @length: the number of bytes to remove.
- * @Returns: the #GByteArray.
  *
  * Removes the given number of bytes starting at the given index from a
  * #GByteArray.  The following elements are moved to close the gap.
  *
+ * Returns: the #GByteArray.
+ *
  * Since: 2.4
  **/
 GByteArray*
