@@ -50,10 +50,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #ifdef HAVE_LANGINFO_TIME
 #include <langinfo.h>
 #endif
@@ -1082,7 +1078,7 @@ g_date_time_add (GDateTime *datetime,
  * @years: the number of years
  *
  * Creates a copy of @datetime and adds the specified number of years to the
- * copy.
+ * copy. Add negative values to subtract years.
  *
  * Return value: the newly created #GDateTime which should be freed with
  *   g_date_time_unref().
@@ -1117,7 +1113,7 @@ g_date_time_add_years (GDateTime *datetime,
  * @months: the number of months
  *
  * Creates a copy of @datetime and adds the specified number of months to the
- * copy.
+ * copy. Add negative values to subtract months.
  *
  * Return value: the newly created #GDateTime which should be freed with
  *   g_date_time_unref().
@@ -1160,7 +1156,7 @@ g_date_time_add_months (GDateTime *datetime,
  * @weeks: the number of weeks
  *
  * Creates a copy of @datetime and adds the specified number of weeks to the
- * copy.
+ * copy. Add negative values to subtract weeks.
  *
  * Return value: the newly created #GDateTime which should be freed with
  *   g_date_time_unref().
@@ -1182,7 +1178,7 @@ g_date_time_add_weeks (GDateTime *datetime,
  * @days: the number of days
  *
  * Creates a copy of @datetime and adds the specified number of days to the
- * copy.
+ * copy. Add negative values to subtract days.
  *
  * Return value: the newly created #GDateTime which should be freed with
  *   g_date_time_unref().
@@ -1206,7 +1202,8 @@ g_date_time_add_days (GDateTime *datetime,
  * @datetime: a #GDateTime
  * @hours: the number of hours to add
  *
- * Creates a copy of @datetime and adds the specified number of hours
+ * Creates a copy of @datetime and adds the specified number of hours.
+ * Add negative values to subtract hours.
  *
  * Return value: the newly created #GDateTime which should be freed with
  *   g_date_time_unref().
@@ -1226,6 +1223,7 @@ g_date_time_add_hours (GDateTime *datetime,
  * @minutes: the number of minutes to add
  *
  * Creates a copy of @datetime adding the specified number of minutes.
+ * Add negative values to subtract minutes.
  *
  * Return value: the newly created #GDateTime which should be freed with
  *   g_date_time_unref().
@@ -1246,6 +1244,7 @@ g_date_time_add_minutes (GDateTime *datetime,
  * @seconds: the number of seconds to add
  *
  * Creates a copy of @datetime and adds the specified number of seconds.
+ * Add negative values to subtract seconds.
  *
  * Return value: the newly created #GDateTime which should be freed with
  *   g_date_time_unref().
@@ -1270,7 +1269,7 @@ g_date_time_add_seconds (GDateTime *datetime,
  * @seconds: the number of seconds to add
  *
  * Creates a new #GDateTime adding the specified values to the current date and
- * time in @datetime.
+ * time in @datetime. Add negative values to subtract.
  *
  * Return value: the newly created #GDateTime that should be freed with
  *   g_date_time_unref().
@@ -2090,6 +2089,59 @@ g_date_time_to_utc (GDateTime *datetime)
 
 /* Format {{{1 */
 
+static gboolean
+format_z (GString *outstr,
+          gint     offset,
+          guint    colons)
+{
+  gint hours;
+  gint minutes;
+  gint seconds;
+
+  hours = offset / 3600;
+  minutes = ABS (offset) / 60 % 60;
+  seconds = ABS (offset) % 60;
+
+  switch (colons)
+    {
+    case 0:
+      g_string_append_printf (outstr, "%+03d%02d",
+                              hours,
+                              minutes);
+      break;
+
+    case 1:
+      g_string_append_printf (outstr, "%+03d:%02d",
+                              hours,
+                              minutes);
+      break;
+
+    case 2:
+      g_string_append_printf (outstr, "%+03d:%02d:%02d",
+                              hours,
+                              minutes,
+                              seconds);
+      break;
+
+    case 3:
+      g_string_append_printf (outstr, "%+03d", hours);
+
+      if (minutes != 0 || seconds != 0)
+        {
+          g_string_append_printf (outstr, ":%02d", minutes);
+
+          if (seconds != 0)
+            g_string_append_printf (outstr, ":%02d", seconds);
+        }
+      break;
+
+    default:
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
 static void
 format_number (GString  *str,
                gboolean  use_alt_digits,
@@ -2185,6 +2237,7 @@ g_date_time_format_locale (GDateTime   *datetime,
 			   gboolean     locale_is_utf8)
 {
   guint     len;
+  guint     colons;
   gchar    *tmp;
   gunichar  c;
   gboolean  alt_digits = FALSE;
@@ -2219,6 +2272,7 @@ g_date_time_format_locale (GDateTime   *datetime,
       if (!*format)
 	break;
 
+      colons = 0;
       alt_digits = FALSE;
       pad_set = FALSE;
 
@@ -2414,17 +2468,15 @@ g_date_time_format_locale (GDateTime   *datetime,
 			 g_date_time_get_year (datetime));
 	  break;
 	case 'z':
-	  if (datetime->tz != NULL)
-	    {
-	      gint64 offset = g_date_time_get_utc_offset (datetime)
-		/ USEC_PER_SECOND;
-
-	      g_string_append_printf (outstr, "%+03d%02d",
-				      (int) offset / 3600,
-				      (int) abs(offset) / 60 % 60);
-	    }
-	  else
-	    g_string_append (outstr, "+0000");
+	  {
+	    gint64 offset;
+	    if (datetime->tz != NULL)
+	      offset = g_date_time_get_utc_offset (datetime) / USEC_PER_SECOND;
+	    else
+	      offset = 0;
+	    if (!format_z (outstr, (int) offset, colons))
+	      return FALSE;
+	  }
 	  break;
 	case 'Z':
 	  tz = g_date_time_get_timezone_abbreviation (datetime);
@@ -2453,6 +2505,12 @@ g_date_time_format_locale (GDateTime   *datetime,
 	  pad_set = TRUE;
 	  pad = "0";
 	  goto next_mod;
+	case ':':
+	  /* Colons are only allowed before 'z' */
+	  if (*format && *format != 'z' && *format != ':')
+	    return FALSE;
+	  colons++;
+	  goto next_mod;
 	default:
 	  return FALSE;
 	}
@@ -2674,7 +2732,23 @@ g_date_time_format_locale (GDateTime   *datetime,
  *  <varlistentry><term>
  *    <literal>\%z</literal>:
  *   </term><listitem><simpara>
- *    the time-zone as hour offset from UTC
+ *    the time zone as an offset from UTC (+hhmm)
+ *  </simpara></listitem></varlistentry>
+ *  <varlistentry><term>
+ *    <literal>\%:z</literal>:
+ *   </term><listitem><simpara>
+ *    the time zone as an offset from UTC (+hh:mm). This is a gnulib strftime extension. Since: 2.38
+ *  </simpara></listitem></varlistentry>
+ *  <varlistentry><term>
+ *    <literal>\%::z</literal>:
+ *   </term><listitem><simpara>
+ *    the time zone as an offset from UTC (+hh:mm:ss). This is a gnulib strftime extension. Since: 2.38
+ *  </simpara></listitem></varlistentry>
+ *  <varlistentry><term>
+ *    <literal>\%:::z</literal>:
+ *   </term><listitem><simpara>
+ *    the time zone as an offset from UTC, with : to necessary precision
+ *    (e.g., -04, +05:30). This is a gnulib strftime extension. Since: 2.38
  *  </simpara></listitem></varlistentry>
  *  <varlistentry><term>
  *    <literal>\%Z</literal>:
