@@ -24,13 +24,13 @@
  * GLib at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#ifndef __G_HASH_H__
+#define __G_HASH_H__
+
 #if !defined (__GLIB_H_INSIDE__) && !defined (GLIB_COMPILATION)
 #error "Only <glib.h> can be included directly."
 #endif
 
-#ifndef __G_HASH_H__
-#define __G_HASH_H__
-
 #include <glib/gtypes.h>
 #include <glib/glist.h>
 
@@ -55,63 +55,94 @@ struct _GHashTableIter
   gpointer      dummy6;
 };
 
+GLIB_AVAILABLE_IN_ALL
 GHashTable* g_hash_table_new               (GHashFunc       hash_func,
                                             GEqualFunc      key_equal_func);
+GLIB_AVAILABLE_IN_ALL
 GHashTable* g_hash_table_new_full          (GHashFunc       hash_func,
                                             GEqualFunc      key_equal_func,
                                             GDestroyNotify  key_destroy_func,
                                             GDestroyNotify  value_destroy_func);
+GLIB_AVAILABLE_IN_ALL
 void        g_hash_table_destroy           (GHashTable     *hash_table);
-void        g_hash_table_insert            (GHashTable     *hash_table,
+GLIB_AVAILABLE_IN_ALL
+gboolean    g_hash_table_insert            (GHashTable     *hash_table,
                                             gpointer        key,
                                             gpointer        value);
-void        g_hash_table_replace           (GHashTable     *hash_table,
+GLIB_AVAILABLE_IN_ALL
+gboolean    g_hash_table_replace           (GHashTable     *hash_table,
                                             gpointer        key,
                                             gpointer        value);
-void        g_hash_table_add               (GHashTable     *hash_table,
+GLIB_AVAILABLE_IN_ALL
+gboolean    g_hash_table_add               (GHashTable     *hash_table,
                                             gpointer        key);
+GLIB_AVAILABLE_IN_ALL
 gboolean    g_hash_table_remove            (GHashTable     *hash_table,
                                             gconstpointer   key);
+GLIB_AVAILABLE_IN_ALL
 void        g_hash_table_remove_all        (GHashTable     *hash_table);
+GLIB_AVAILABLE_IN_ALL
 gboolean    g_hash_table_steal             (GHashTable     *hash_table,
                                             gconstpointer   key);
+GLIB_AVAILABLE_IN_ALL
 void        g_hash_table_steal_all         (GHashTable     *hash_table);
+GLIB_AVAILABLE_IN_ALL
 gpointer    g_hash_table_lookup            (GHashTable     *hash_table,
                                             gconstpointer   key);
+GLIB_AVAILABLE_IN_ALL
 gboolean    g_hash_table_contains          (GHashTable     *hash_table,
-                                            gconstpointer   lookup_key);
+                                            gconstpointer   key);
+GLIB_AVAILABLE_IN_ALL
 gboolean    g_hash_table_lookup_extended   (GHashTable     *hash_table,
                                             gconstpointer   lookup_key,
                                             gpointer       *orig_key,
                                             gpointer       *value);
+GLIB_AVAILABLE_IN_ALL
 void        g_hash_table_foreach           (GHashTable     *hash_table,
                                             GHFunc          func,
                                             gpointer        user_data);
+GLIB_AVAILABLE_IN_ALL
 gpointer    g_hash_table_find              (GHashTable     *hash_table,
                                             GHRFunc         predicate,
                                             gpointer        user_data);
+GLIB_AVAILABLE_IN_ALL
 guint       g_hash_table_foreach_remove    (GHashTable     *hash_table,
                                             GHRFunc         func,
                                             gpointer        user_data);
+GLIB_AVAILABLE_IN_ALL
 guint       g_hash_table_foreach_steal     (GHashTable     *hash_table,
                                             GHRFunc         func,
                                             gpointer        user_data);
+GLIB_AVAILABLE_IN_ALL
 guint       g_hash_table_size              (GHashTable     *hash_table);
+GLIB_AVAILABLE_IN_ALL
 GList *     g_hash_table_get_keys          (GHashTable     *hash_table);
+GLIB_AVAILABLE_IN_ALL
 GList *     g_hash_table_get_values        (GHashTable     *hash_table);
+GLIB_AVAILABLE_IN_2_40
+gpointer *  g_hash_table_get_keys_as_array (GHashTable     *hash_table,
+                                            guint          *length);
 
+GLIB_AVAILABLE_IN_ALL
 void        g_hash_table_iter_init         (GHashTableIter *iter,
                                             GHashTable     *hash_table);
+GLIB_AVAILABLE_IN_ALL
 gboolean    g_hash_table_iter_next         (GHashTableIter *iter,
                                             gpointer       *key,
                                             gpointer       *value);
+GLIB_AVAILABLE_IN_ALL
 GHashTable* g_hash_table_iter_get_hash_table (GHashTableIter *iter);
+GLIB_AVAILABLE_IN_ALL
 void        g_hash_table_iter_remove       (GHashTableIter *iter);
+GLIB_AVAILABLE_IN_2_30
 void        g_hash_table_iter_replace      (GHashTableIter *iter,
                                             gpointer        value);
+GLIB_AVAILABLE_IN_ALL
 void        g_hash_table_iter_steal        (GHashTableIter *iter);
 
+GLIB_AVAILABLE_IN_ALL
 GHashTable* g_hash_table_ref               (GHashTable     *hash_table);
+GLIB_AVAILABLE_IN_ALL
 void        g_hash_table_unref             (GHashTable     *hash_table);
 
 #ifndef G_DISABLE_DEPRECATED
@@ -121,23 +152,33 @@ void        g_hash_table_unref             (GHashTable     *hash_table);
 
 /* Hash Functions
  */
+GLIB_AVAILABLE_IN_ALL
 gboolean g_str_equal    (gconstpointer  v1,
                          gconstpointer  v2);
+GLIB_AVAILABLE_IN_ALL
 guint    g_str_hash     (gconstpointer  v);
 
+GLIB_AVAILABLE_IN_ALL
 gboolean g_int_equal    (gconstpointer  v1,
                          gconstpointer  v2);
+GLIB_AVAILABLE_IN_ALL
 guint    g_int_hash     (gconstpointer  v);
 
+GLIB_AVAILABLE_IN_ALL
 gboolean g_int64_equal  (gconstpointer  v1,
                          gconstpointer  v2);
+GLIB_AVAILABLE_IN_ALL
 guint    g_int64_hash   (gconstpointer  v);
 
+GLIB_AVAILABLE_IN_ALL
 gboolean g_double_equal (gconstpointer  v1,
                          gconstpointer  v2);
+GLIB_AVAILABLE_IN_ALL
 guint    g_double_hash  (gconstpointer  v);
 
+GLIB_AVAILABLE_IN_ALL
 guint    g_direct_hash  (gconstpointer  v) G_GNUC_CONST;
+GLIB_AVAILABLE_IN_ALL
 gboolean g_direct_equal (gconstpointer  v1,
                          gconstpointer  v2) G_GNUC_CONST;
 
