@@ -27,7 +27,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#ifdef HAVE_UNISTD_H
+#ifdef G_OS_UNIX
 #include <unistd.h>
 #endif
 
@@ -46,7 +46,7 @@
 #include "gstdio.h"
 
 
-#if !defined (G_OS_UNIX) && !defined (G_OS_WIN32) && !defined (G_OS_BEOS)
+#if !defined (G_OS_UNIX) && !defined (G_OS_WIN32)
 #error Please port this to your operating system
 #endif
 
@@ -848,6 +848,8 @@ g_utime (const gchar    *filename,
  * function over the call provided by the system; on Unix, it will
  * attempt to correctly handle %EINTR, which has platform-specific
  * semantics.
+ *
+ * Since: 2.36
  */
 gboolean
 g_close (gint       fd,
