@@ -865,7 +865,7 @@ g_ascii_strtod (const gchar *nptr,
  * Converts a #gdouble to a string, using the '.' as
  * decimal point.
  *
- * This functions generates enough precision that converting
+ * This function generates enough precision that converting
  * the string back using g_ascii_strtod() gives the same machine-number
  * (on machines with IEEE compatible 64bit doubles). It is
  * guaranteed that the size of the resulting string will never
@@ -1250,25 +1250,14 @@ g_ascii_strtoll (const gchar *nptr,
 const gchar *
 g_strerror (gint errnum)
 {
-  gchar buf[64];
   gchar *msg;
-  gchar *tofree;
+  gchar *tofree = NULL;
   const gchar *ret;
   gint saved_errno = errno;
 
-  msg = tofree = NULL;
-
-#ifdef HAVE_STRERROR
   msg = strerror (errnum);
   if (!g_get_charset (NULL))
     msg = tofree = g_locale_to_utf8 (msg, -1, NULL, NULL, NULL);
-#endif
-
-  if (!msg)
-    {
-      msg = buf;
-      _g_sprintf (msg, "unknown error (%d)", errnum);
-    }
 
   ret = g_intern_string (msg);
   g_free (tofree);
@@ -1761,6 +1750,8 @@ g_ascii_xdigit_value (gchar c)
  * characters include all ASCII letters. If you compare two CP932
  * strings using this function, you will get false matches.
  *
+ * Both @s1 and @s2 must be non-%NULL.
+ *
  * Return value: 0 if the strings match, a negative value if @s1 &lt; @s2,
  *   or a positive value if @s1 &gt; @s2.
  **/
@@ -2218,7 +2209,7 @@ g_strchug (gchar *string)
   for (start = (guchar*) string; *start && g_ascii_isspace (*start); start++)
     ;
 
-  g_memmove (string, start, strlen ((gchar *) start) + 1);
+  memmove (string, start, strlen ((gchar *) start) + 1);
 
   return string;
 }
