@@ -103,6 +103,9 @@ GLIB_AVAILABLE_IN_ALL
 GVariant *                      g_variant_new_string                    (const gchar          *string);
 GLIB_AVAILABLE_IN_2_38
 GVariant *                      g_variant_new_take_string               (gchar                *string);
+GLIB_AVAILABLE_IN_2_38
+GVariant *                      g_variant_new_printf                    (const gchar          *format_string,
+                                                                         ...) G_GNUC_PRINTF (1, 2);
 GLIB_AVAILABLE_IN_ALL
 GVariant *                      g_variant_new_object_path               (const gchar          *object_path);
 GLIB_AVAILABLE_IN_ALL
@@ -320,10 +323,13 @@ typedef enum
   G_VARIANT_PARSE_ERROR_UNTERMINATED_STRING_CONSTANT,
   G_VARIANT_PARSE_ERROR_VALUE_EXPECTED
 } GVariantParseError;
-#define G_VARIANT_PARSE_ERROR (g_variant_parser_get_error_quark ())
+#define G_VARIANT_PARSE_ERROR (g_variant_parse_error_quark ())
+
+GLIB_DEPRECATED_IN_2_38_FOR(g_variant_parse_error_quark)
+GQuark                          g_variant_parser_get_error_quark        (void);
 
 GLIB_AVAILABLE_IN_ALL
-GQuark                          g_variant_parser_get_error_quark        (void);
+GQuark                          g_variant_parse_error_quark             (void);
 
 GLIB_AVAILABLE_IN_ALL
 GVariantBuilder *               g_variant_builder_new                   (const GVariantType   *type);
@@ -389,6 +395,10 @@ GLIB_AVAILABLE_IN_ALL
 GVariant *                      g_variant_new_parsed_va                 (const gchar          *format,
                                                                          va_list              *app);
 
+GLIB_AVAILABLE_IN_2_40
+gchar *                         g_variant_parse_error_print_context     (GError               *error,
+                                                                         const gchar          *source_str);
+
 GLIB_AVAILABLE_IN_ALL
 gint                            g_variant_compare                       (gconstpointer one,
                                                                          gconstpointer two);
