@@ -10,10 +10,16 @@ typedef struct {
 } CollateTest;
 
 typedef struct {
-  const gchar *key;
+  gchar *key;
   const gchar *str;
 } Line;
 
+static void
+clear_line (Line *line)
+{
+  g_free (line->key);
+}
+
 static int
 compare_collate (const void *a, const void *b)
 {
@@ -35,10 +41,13 @@ compare_key (const void *a, const void *b)
 static void
 do_collate (gboolean for_file, gboolean use_key, const CollateTest *test)
 {
-  GArray *line_array = g_array_new (FALSE, FALSE, sizeof(Line));
+  GArray *line_array;
   Line line;
   gint i;
 
+  line_array = g_array_new (FALSE, FALSE, sizeof(Line));
+  g_array_set_clear_func (line_array, (GDestroyNotify)clear_line);
+
   for (i = 0; test->input[i]; i++)
     {
       line.str = test->input[i];
@@ -61,6 +70,8 @@ do_collate (gboolean for_file, gboolean use_key, const CollateTest *test)
       else
         g_assert_cmpstr (str, ==, test->sorted[i]);
     }
+
+  g_array_free (line_array, TRUE);
 }
 
 static void
@@ -177,13 +188,64 @@ const gchar *file_sorted1[] = {
   NULL
 };
 
+const gchar *input2[] = {
+  "file26",
+  "file100",
+  "file1",
+  "file:foo",
+  "a.a",
+  "file027",
+  "file10",
+  "aa.a",
+  "file5",
+  "file0027",
+  "a-.a",
+  "file0000",
+  "file000x",
+  NULL
+};
+
+const gchar *sorted2[] = {
+  "a-.a",
+  "a.a",
+  "aa.a",
+  "file0000",
+  "file000x",
+  "file0027",
+  "file027",
+  "file1",
+  "file10",
+  "file100",
+  "file26",
+  "file5",
+  "file:foo",
+  NULL
+};
+
+const gchar *file_sorted2[] = {
+  "a.a",
+  "a-.a",
+  "aa.a",
+  "file0000",
+  "file000x",
+  "file1",
+  "file5",
+  "file10",
+  "file26",
+  "file027",
+  "file0027",
+  "file100",
+  "file:foo",
+  NULL
+};
+
 int
 main (int argc, char *argv[])
 {
   gchar *path;
   gint i;
   const gchar *locale;
-  CollateTest test[2];
+  CollateTest test[3];
 
   g_test_init (&argc, &argv, NULL);
 
@@ -201,6 +263,9 @@ main (int argc, char *argv[])
   test[1].input = input1;
   test[1].sorted = sorted1;
   test[1].file_sorted = file_sorted1;
+  test[2].input = input2;
+  test[2].sorted = sorted2;
+  test[2].file_sorted = file_sorted2;
 
   for (i = 0; i < G_N_ELEMENTS (test); i++)
     {
@@ -208,10 +273,10 @@ main (int argc, char *argv[])
       g_test_add_data_func (path, &test[i], test_collate);
       g_free (path);
       path = g_strdup_printf ("/unicode/collate-key/%d", i);
-      g_test_add_data_func (path, test, test_collate_key);
+      g_test_add_data_func (path, &test[i], test_collate_key);
       g_free (path);
       path = g_strdup_printf ("/unicode/collate-filename/%d", i);
-      g_test_add_data_func (path, test, test_collate_file);
+      g_test_add_data_func (path, &test[i], test_collate_file);
       g_free (path);
     }
 
