@@ -1,10 +1,5 @@
-#include "config.h"
-
 #include <glib.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #include <stdlib.h>
 
 /* HMAC-MD5 test vectors as per RFC 2202 */
