@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /* A test program for the main loop and IO channel code.
@@ -214,11 +216,11 @@ recv_windows_message (GIOChannel  *channel,
 {
   GIOError error;
   MSG msg;
-  gsize nb;
+  guint nb;
   
   while (1)
     {
-      error = g_io_channel_read (channel, (gchar *) &msg, sizeof (MSG), &nb);
+      error = g_io_channel_read (channel, &msg, sizeof (MSG), &nb);
       
       if (error != G_IO_ERROR_NONE)
 	{
@@ -232,25 +234,20 @@ recv_windows_message (GIOChannel  *channel,
       break;
     }
 
-  g_print ("gio-test: ...Windows message for 0x%p: %d,%" G_GUINTPTR_FORMAT ",%" G_GINTPTR_FORMAT "\n",
-	   msg.hwnd, msg.message, msg.wParam, (gintptr)msg.lParam);
+  g_print ("gio-test: ...Windows message for %#x: %d,%d,%d\n",
+	   msg.hwnd, msg.message, msg.wParam, msg.lParam);
 
   return TRUE;
 }
 
-LRESULT CALLBACK window_procedure (HWND   hwnd,
-                                   UINT   message,
-                                   WPARAM wparam,
-                                   LPARAM lparam);
-
 LRESULT CALLBACK 
 window_procedure (HWND hwnd,
 		  UINT message,
 		  WPARAM wparam,
 		  LPARAM lparam)
 {
-  g_print ("gio-test: window_procedure for 0x%p: %d,%" G_GUINTPTR_FORMAT ",%" G_GINTPTR_FORMAT "\n",
-	   hwnd, message, wparam, (gintptr)lparam);
+  g_print ("gio-test: window_procedure for %#x: %d,%d,%d\n",
+	   hwnd, message, wparam, lparam);
   return DefWindowProc (hwnd, message, wparam, lparam);
 }
 
@@ -336,7 +333,7 @@ main (int    argc,
 	  nrunning++;
 	  
 #ifdef G_OS_WIN32
-	  cmdline = g_strdup_printf ("%d:%d:0x%p",
+	  cmdline = g_strdup_printf ("%d:%d:%d",
 				     pipe_to_sub[0],
 				     pipe_from_sub[1],
 				     hwnd);
@@ -393,7 +390,7 @@ main (int    argc,
       sscanf (argv[2], "%d:%d%n", &readfd, &writefd, &n);
 
 #ifdef G_OS_WIN32
-      sscanf (argv[2] + n, ":0x%p", &hwnd);
+      sscanf (argv[2] + n, ":%d", &hwnd);
 #endif
       
       srand (tv.tv_sec ^ (tv.tv_usec / 1000) ^ readfd ^ (writefd << 4));
@@ -418,8 +415,8 @@ main (int    argc,
 	      int msg = WM_USER + (rand() % 100);
 	      WPARAM wparam = rand ();
 	      LPARAM lparam = rand ();
-	      g_print ("gio-test: child posting message %d,%" G_GUINTPTR_FORMAT ",%" G_GINTPTR_FORMAT " to 0x%p\n",
-		       msg, wparam, (gintptr)lparam, hwnd);
+	      g_print ("gio-test: child posting message %d,%d,%d to %#x\n",
+		       msg, wparam, lparam, hwnd);
 	      PostMessage (hwnd, msg, wparam, lparam);
 	    }
 #endif
